package test;

import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * Bean implementation class for Enterprise Bean: PSPSim
 */
public class PSPSimBean implements javax.ejb.SessionBean {
	private javax.ejb.SessionContext mySessionCtx;
	/**
	 * getSessionContext
	 */
	public javax.ejb.SessionContext getSessionContext() {
		return mySessionCtx;
	}
	/**
	 * setSessionContext
	 */
	public void setSessionContext(javax.ejb.SessionContext ctx) {
		mySessionCtx = ctx;
	}
	/**
	 * ejbCreate
	 */
	public void ejbCreate() throws javax.ejb.CreateException {
	}
	/**
	 * ejbActivate
	 */
	public void ejbActivate() {
	}
	/**
	 * ejbPassivate
	 */
	public void ejbPassivate() {
	}
	/**
	 * ejbRemove
	 */
	public void ejbRemove() {
	}
	public void updateCardStatus(CardData cardData) {
		try {
			InitialContext ic = new InitialContext();
			CardStatusLocalHome cardStatusHome = (CardStatusLocalHome)ic.lookup("java:comp/env/ejb/CardStatus");
			try {
				CardStatusLocal cardStatus = cardStatusHome.findByPrimaryKey(new CardStatusKey(cardData.getCardBrand(), cardData.getCardNumber()));
				cardStatus.setCardData(cardData);
			}
			catch(ObjectNotFoundException e) {
				cardStatusHome.create(cardData);
			}
		}
		catch(CreateException e) {
			throw new EJBException(e);
		}
		catch(FinderException e) {
			throw new EJBException(e);
		}
		catch(NamingException e) {
			throw new EJBException(e);
		}
	}
	public void deleteCardStatus(String cardBrand, String cardNumber) {
		try {
			InitialContext ic = new InitialContext();
			CardStatusLocalHome cardStatusHome = (CardStatusLocalHome)ic.lookup("java:comp/env/ejb/CardStatus");
			CardStatusLocal cardStatus = cardStatusHome.findByPrimaryKey(new CardStatusKey(cardBrand, cardNumber));
			cardStatus.remove();
		}
		catch(FinderException e) {
			throw new EJBException(e);
		}
		catch(NamingException e) {
			throw new EJBException(e);
		}
		catch(RemoveException e) {
			throw new EJBException(e);
		}
	}
	public CardData getCardStatus(String cardBrand, String cardNumber) {
		try {
			InitialContext ic = new InitialContext();
			CardStatusLocalHome cardStatusHome = (CardStatusLocalHome)ic.lookup("java:comp/env/ejb/CardStatus");
			CardStatusLocal cardStatus = cardStatusHome.findByPrimaryKey(new CardStatusKey(cardBrand, cardNumber));
			return cardStatus.getCardData();
		}
		catch(ObjectNotFoundException e) {
			return null;
		}
		catch(FinderException e) {
			throw new EJBException(e);
		}
		catch(NamingException e) {
			throw new EJBException(e);
		}
	}
	public TransactionData processTransaction(TransactionData transactionData) {
		try {
			transactionData.setTimestamp(new Date());
			InitialContext ic = new InitialContext();
			CardStatusLocalHome cardStatusHome = (CardStatusLocalHome)ic.lookup("java:comp/env/ejb/CardStatus");
			TransactionRecordLocalHome txRecordHome = (TransactionRecordLocalHome)ic.lookup("java:comp/env/ejb/TransactionRecord");
			try {
				CardStatusLocal cardStatus = cardStatusHome.findByPrimaryKey(new CardStatusKey(transactionData.getCardBrand(), transactionData.getCardNumber()));
				if(!transactionData.getExpiryDate().equals(cardStatus.getExpiryDate())) {
					transactionData.setReturnCode("1001");
				}
				else if(!cardStatus.getStatus().equals("A")) {
					transactionData.setReturnCode("1002");
				}
				else if(!transactionData.getCurrency().equals(cardStatus.getCurrency())) {
					transactionData.setReturnCode("2001");
				}
				else if(transactionData.getTransactionType().equals("A")) {
					if(transactionData.getAmount().compareTo(cardStatus.getCreditLimit()) > 0) {
						transactionData.setReturnCode("2002");
					}
					else {
						cardStatus.setCreditLimit(cardStatus.getCreditLimit().subtract(transactionData.getAmount()));
						transactionData.setReturnCode("0000");
						transactionData.setAuthorizationNumber(Long.toString(transactionData.getTimestamp().getTime() % 10000000000l));
					}
				}
				else if(transactionData.getTransactionType().equals("D")) {
					Collection txRecords = txRecordHome.findByMerchantIdAndAuthorizationNumber(transactionData.getMerchantId(), transactionData.getAuthorizationNumber());
					if(txRecords.size() == 0) {
						transactionData.setReturnCode("2003");
					}
					else if(txRecords.size() > 1) {
						transactionData.setReturnCode("2004");
					}
					else {
						TransactionRecordLocal txRecord = (TransactionRecordLocal)txRecords.iterator().next();
						if(!txRecord.getTransactionType().equals("A")) {
							transactionData.setReturnCode("2005");
						}
						else if(!transactionData.getCardBrand().equals(txRecord.getCardBrand())
								|| !transactionData.getCardNumber().trim().equals(txRecord.getCardNumber().trim())) {
							transactionData.setReturnCode("2006");
						}
						else if(transactionData.getAmount().compareTo(txRecord.getAmount()) != 0) {
							transactionData.setReturnCode("2007");
						}
						else {
							transactionData.setReturnCode("0000");
						}
					}
				}
			}
			catch(ObjectNotFoundException e) {
				transactionData.setReturnCode("1003");
			}
			try {
				txRecordHome.create(transactionData);
			}
			catch(DuplicateKeyException e) {
				transactionData.setReturnCode("3001");
			}
			return transactionData;
		}
		catch(CreateException e) {
			throw new EJBException(e);
		}
		catch(FinderException e) {
			throw new EJBException(e);
		}
		catch(NamingException e) {
			throw new EJBException(e);
		}
	}
	public TransactionData getTransactionRecord(String merchantId, String transactionId) {
		try {
			InitialContext ic = new InitialContext();
			TransactionRecordLocalHome txRecordHome = (TransactionRecordLocalHome)ic.lookup("java:comp/env/ejb/TransactionRecord");
			TransactionRecordLocal txRecord = txRecordHome.findByPrimaryKey(new TransactionRecordKey(merchantId, transactionId));
			return txRecord.getTransactionData();
		}
		catch(ObjectNotFoundException e) {
			return null;
		}
		catch(FinderException e) {
			throw new EJBException(e);
		}
		catch(NamingException e) {
			throw new EJBException(e);
		}
	}
	public TransactionData[] getTransactionRecordsByMerchantId(String merchantId) {
		try {
			InitialContext ic = new InitialContext();
			TransactionRecordLocalHome txRecordHome = (TransactionRecordLocalHome)ic.lookup("java:comp/env/ejb/TransactionRecord");
			Collection txRecords = txRecordHome.findbyMerchantId(merchantId);
			Iterator it = txRecords.iterator();
			TransactionData[] txData = new TransactionData[txRecords.size()];
			for(int i = 0; it.hasNext(); i++) {
				TransactionRecordLocal txRecord = (TransactionRecordLocal)it.next();
				txData[i] = txRecord.getTransactionData();
			}
			return txData;
		}
		catch(ObjectNotFoundException e) {
			return null;
		}
		catch(FinderException e) {
			throw new EJBException(e);
		}
		catch(NamingException e) {
			throw new EJBException(e);
		}
	}
	public TransactionData[] getTransactionRecordsByCardBrandAndCardNumber(String cardBrand, String cardNumber) {
		try {
			InitialContext ic = new InitialContext();
			TransactionRecordLocalHome txRecordHome = (TransactionRecordLocalHome)ic.lookup("java:comp/env/ejb/TransactionRecord");
			Collection txRecords = txRecordHome.findByCardBrandAndCardNumber(cardBrand, cardNumber);
			Iterator it = txRecords.iterator();
			TransactionData[] txData = new TransactionData[txRecords.size()];
			for(int i = 0; it.hasNext(); i++) {
				TransactionRecordLocal txRecord = (TransactionRecordLocal)it.next();
				txData[i] = txRecord.getTransactionData();
			}
			return txData;
		}
		catch(ObjectNotFoundException e) {
			return null;
		}
		catch(FinderException e) {
			throw new EJBException(e);
		}
		catch(NamingException e) {
			throw new EJBException(e);
		}
	}
}
